"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var path = _interopRequireWildcard(require("path"));

var _express = _interopRequireDefault(require("express"));

var _cookieParser = _interopRequireDefault(require("cookie-parser"));

var _utils = require("@modern-js/utils");

var _finalhandler = _interopRequireDefault(require("finalhandler"));

var _context = require("./context");

var _registerRoutes = _interopRequireDefault(require("./registerRoutes"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const debug = (0, _utils.createDebugger)('express');

const findAppModule = async apiDir => {
  const exts = ['.ts', '.js'];
  const paths = exts.map(ext => path.resolve(apiDir, `app${ext}`));

  for (const filename of paths) {
    if (await _utils.fs.pathExists(filename)) {
      // 每次获取 app.ts 的时候，避免使用缓存的 app.ts
      delete require.cache[filename];
      return [(0, _utils.compatRequire)(filename), require(filename)];
    }
  }

  return [];
};

const initMiddlewares = (middleware, app) => {
  middleware.forEach(middlewareItem => {
    const middlewareFunc = typeof middlewareItem === 'string' ? (0, _utils.compatRequire)(middlewareItem) : middlewareItem;
    app.use(middlewareFunc);
  });
};

const useRun = app => {
  app.use((req, res, next) => {
    (0, _context.run)({
      req,
      res
    }, next);
  });
};

const initApp = app => {
  app.use((0, _cookieParser.default)());
  app.use(_express.default.text());
  app.use(_express.default.urlencoded({
    extended: true
  }));
  app.use(_express.default.json());
  return app;
};

var _default = () => ({
  name: '@modern-js/plugin-express',
  pre: ['@modern-js/plugin-bff'],
  setup: api => ({
    async prepareApiServer({
      pwd,
      config
    }) {
      let app;
      const apiDir = path.join(pwd, './api');
      const appContext = api.useAppContext();
      const apiHandlerInfos = appContext.apiHandlerInfos;
      const mode = appContext.apiMode;

      if (mode === 'framework') {
        const appModule = await findAppModule(apiDir);
        app = appModule[0];
        const hooks = appModule[1];

        if (!app || !app.use) {
          // console.warn('There is not api/app.ts.');
          app = (0, _express.default)();
        }

        initApp(app);

        if (config) {
          const {
            middleware
          } = config;
          initMiddlewares(middleware, app);
        }

        useRun(app);
        (0, _registerRoutes.default)(app, apiHandlerInfos);

        if (hooks) {
          const {
            afterLambdaRegisted
          } = hooks;

          if (afterLambdaRegisted) {
            afterLambdaRegisted(app);
          }
        }
      } else if (mode === 'function') {
        app = (0, _express.default)();
        initApp(app);

        if (config) {
          const {
            middleware
          } = config;
          initMiddlewares(middleware, app);
        }

        useRun(app);
        (0, _registerRoutes.default)(app, apiHandlerInfos);
      } else {
        throw new Error(`mode must be function or framework`);
      }

      return (req, res) => new Promise((resolve, reject) => {
        const handler = err => {
          if (err) {
            return reject(err);
          } // finalhanlder will trigger 'finish' event


          return (0, _finalhandler.default)(req, res, {})(null); // return resolve();
        };

        res.on('finish', err => {
          if (err) {
            return reject(err);
          }

          return resolve();
        });
        return app(req, res, handler);
      });
    },

    prepareWebServer({
      config
    }) {
      const app = (0, _express.default)();
      initApp(app);

      if (config) {
        const {
          middleware
        } = config;
        debug('web middleware', middleware);
        initMiddlewares(middleware, app);
      }

      return (req, res) => new Promise((resolve, reject) => {
        const handler = err => {
          if (err) {
            return reject(err);
          }

          if (res.headersSent && res.statusCode !== 200) {
            (0, _finalhandler.default)(req, res, {})(null);
          }

          return resolve();
        }; // when user call res.send


        res.on('finish', err => {
          if (err) {
            return reject(err);
          }

          return resolve();
        });
        return app(req, res, handler);
      });
    }

  })
});

exports.default = _default;