"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isNormalMethod = exports.createRouteHandler = void 0;

require("reflect-metadata");

var _bffCore = require("@modern-js/bff-core");

var _bffRuntime = require("@modern-js/bff-runtime");

var _typeIs = _interopRequireDefault(require("type-is"));

var _formidable = _interopRequireDefault(require("formidable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const handleResponseMeta = (res, handler) => {
  const responseMeta = Reflect.getMetadata(_bffCore.HttpMetadata.Response, handler);

  if (Array.isArray(responseMeta)) {
    for (const meta of responseMeta) {
      const metaType = meta.type;
      const metaValue = meta.value;

      switch (metaType) {
        case _bffCore.ResponseMetaType.Headers:
          // eslint-disable-next-line @typescript-eslint/ban-types
          for (const [key, value] of Object.entries(metaValue)) {
            res.append(key, value);
          }

          break;

        case _bffCore.ResponseMetaType.Redirect:
          res.redirect(metaValue);
          break;

        case _bffCore.ResponseMetaType.StatusCode:
          res.status(metaValue);
          break;

        default:
          break;
      }
    }
  }
};

const createRouteHandler = handler => {
  const apiHandler = async (req, res, next) => {
    const input = await getInputFromRequest(req);

    if ((0, _bffCore.isWithMetaHandler)(handler)) {
      try {
        handleResponseMeta(res, handler);

        if (res.headersSent) {
          return;
        }

        const result = await handler(input);

        if (result && typeof result === 'object') {
          // eslint-disable-next-line consistent-return
          return res.json(result);
        }
      } catch (error) {
        if (error instanceof _bffCore.ValidationError) {
          res.status(error.status); // eslint-disable-next-line consistent-return

          return res.json({
            message: error.message
          });
        }

        throw error;
      }
    } else if ((0, _bffRuntime.isSchemaHandler)(handler)) {
      const result = await handler(input);

      if (result.type !== 'HandleSuccess') {
        if (result.type === 'InputValidationError') {
          res.status(400);
        } else {
          res.status(500);
        } // eslint-disable-next-line consistent-return


        return res.json(result.message);
      } else {
        res.status(200); // eslint-disable-next-line consistent-return

        return res.json(result.value);
      }
    } else {
      const args = Object.values(input.params).concat(input);

      try {
        const body = await handler(...args); // this should never happen

        if (res.headersSent) {
          // eslint-disable-next-line consistent-return
          return await Promise.resolve();
        }

        if (typeof body !== 'undefined') {
          // eslint-disable-next-line consistent-return
          return res.json(body);
        }
      } catch (e) {
        // eslint-disable-next-line consistent-return
        return next(e);
      }
    }
  };

  Object.defineProperties(apiHandler, Object.getOwnPropertyDescriptors(handler));
  return apiHandler;
};

exports.createRouteHandler = createRouteHandler;

const isNormalMethod = httpMethod => _bffCore.httpMethods.includes(httpMethod);

exports.isNormalMethod = isNormalMethod;

const getInputFromRequest = async request => {
  const draft = {
    params: request.params,
    query: request.query,
    headers: request.headers,
    cookies: request.headers.cookie
  };

  if ((0, _typeIs.default)(request, ['application/json'])) {
    draft.data = request.body;
  } else if ((0, _typeIs.default)(request, ['multipart/form-data'])) {
    draft.formData = await resolveFormData(request);
  } else if ((0, _typeIs.default)(request, ['application/x-www-form-urlencoded'])) {
    draft.formUrlencoded = request.body;
  } else {
    draft.body = request.body;
  }

  return draft;
};

const resolveFormData = request => {
  const form = (0, _formidable.default)({
    multiples: true
  });
  return new Promise((resolve, reject) => {
    form.parse(request, (err, fields, files) => {
      if (err) {
        reject(err);
      }

      resolve(_objectSpread(_objectSpread({}, fields), files));
    });
  });
};